// Virtuino client code to communicate with Virtuino Server created with VirtuinoCM library
// Code for all ESP8266 modules
// Created by Ilias Lamprou
// Feb/05/2020

#include <ESP8266WiFi.h>

//-----------------SETTINGS -------------------
const char* ssid     = "WIFI NETWORK";
const char* password = "PASSWORD";

const char* virtuinoServerIP = "192.168.1.150";  // The IP of the board that runs the VirtuinoCM server code
int virtuinoServerPort = 8000;                   // The Por of the board that runs the VirtuinoCM server code
const char * virtuinoKey = "1234";
//---------------------------------------------

String commandBuffer="";



//==================================================== setup
//====================================================
void setup() {
  Serial.begin(9600);
  delay(10);
  Serial.println();
  commandBuffer.reserve(256);     // increase this if needed
  //-- Code for the connection to the WiFi
  Serial.print("Connecting to ");
  Serial.println(ssid);
  WiFi.begin(ssid, password);
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }
  Serial.println("");
  Serial.println("WiFi connected");  
  Serial.println("IP address: ");
  Serial.println(WiFi.localIP());


  pinMode(D4, OUTPUT);
}



//==================================================== loop
//====================================================
void loop() {

  //----------- 1. Add code here to read your sensor values
   int value1 = analogRead(A0);           // Replace with the value you want to send to virtuino
   int value2 = random(100);              // Replace with the value you want to send to virtuino


   // ----------- 2. Create the command String
   commandBuffer="";                      // clear buffer before you send new command to VirtuinoCM library
   commandBuffer+=virtuinoKey;            // start the command with the Virtuino server key
   commandBuffer+="!V1=?$";                  // add a command to get the value of the V1
   commandBuffer+="!V2=?$";                  // add a command to get the value of the V2
   commandBuffer+="!V3=?$";                  // add a command to get the value of the V3
   //commandBuffer+="!V10=23.43$";           // add a command to send a value to the V10

   //----------- 3. Send the command to VirtuinoCM library   
   if (sendCommand())  {
    Serial.println("Command is sent succesfully");
    float val1 = getValue('V', 1); 
    Serial.println("V1="+String(val1));

    float val2 = getValue('V', 2); 
    Serial.println("V2="+String(val2));

    float val3 = getValue('V', 3); 
    Serial.println("V3="+String(val3));
   }
   else Serial.println("Error. Can't send command");

   delay (5000);  // send command every 10 sec

}

//==================================================== sendCommand
//====================================================
boolean sendCommand(){
  if (commandBuffer.length()==0) return false;
  Serial.println("Sending the text: "+commandBuffer);
  Serial.print("Connecting to ");
  Serial.println(virtuinoServerIP);
  
  WiFiClient client;
    if (!client.connect(virtuinoServerIP,virtuinoServerPort)) {
    Serial.println("Connection failed");
    return false;
  } else Serial.println("Connected!");
  
  // This will send the request to the server
  client.print(commandBuffer);
  unsigned long timeout = millis();
  while (client.available() == 0) {
    if (millis() - timeout > 5000) {
      Serial.println(">>> Client Timeout !");
      client.stop();
      return false;
    }
  }
  
  // Read all the lines of the reply from server and print them to Serial
  commandBuffer="";
  while(client.available()){
      char ch = static_cast<char>(client.read());
      Serial.print(ch);
      commandBuffer+=ch;
  }
  Serial.println("/nResponse:"+commandBuffer);
  Serial.println("closing connection");
  return true;
  }

//======================================================================== getResponseValue
//========================================================================
float getValue(char pinType, byte pin){
  String s="!";
  s+=pinType;
  s+=String(pin)+"=";
  int pos= commandBuffer.indexOf(s);     // the value is existed on the replied string
  Serial.println("pos1: "+String(pos));
  if (pos!=-1) {
    pos= commandBuffer.indexOf("=",pos);
    int lastChar = commandBuffer.indexOf('$',pos);
    if (lastChar!=-1){
      String valueString = commandBuffer.substring(pos+1,lastChar);
      valueString.trim();
      return valueString.toFloat();
    }
  }
  return NULL;
}
